@@ -33,6 +33,7 @@
 #include <sys/stat.h>
 
 #include "config.h"
+#include "match.h"
 
 
 #define ST_GROUND 0
@@ -110,7 +111,7 @@ not_permitted( char* directive, char* tag, char* val )
     (void) printf( "\
 <HR><H2>%s</H2>\n\
 The filename requested in the %s %s=%s directive\n\
-contains .. or is an absolute pathname.\n\
+may not be fetched.\n\
 <HR>\n", title, directive, tag, val );
     }
 
@@ -213,6 +214,74 @@ get_filename( char* vfilename, char* filename, char* directive, char* tag, char*
     }
 
 
+static int
+check_filename( char* filename )
+    {
+    static int inited = 0;
+    static char* cgi_pattern;
+    int fnl;
+    char* cp;
+    char* dirname;
+    char* authname;
+    struct stat sb;
+    int r;
+
+    if ( ! inited )
+	{
+	/* Get the cgi pattern. */
+	cgi_pattern = getenv( "CGI_PATTERN" );
+#ifdef CGI_PATTERN
+	if ( cgi_pattern == (char*) 0 )
+	    cgi_pattern = CGI_PATTERN;
+#endif /* CGI_PATTERN */
+	inited = 1;
+	}
+
+    /* ../ is not permitted. */
+    if ( strstr( filename, "../" ) != (char*) 0 )
+	return 0;
+
+#ifdef AUTH_FILE
+    /* Ensure that we are not reading a basic auth password file. */
+    fnl = strlen(filename);
+    if ( strcmp( filename, AUTH_FILE ) == 0 ||
+	 ( fnl >= sizeof(AUTH_FILE) &&
+	   strcmp( &filename[fnl - sizeof(AUTH_FILE) + 1], AUTH_FILE ) == 0 &&
+	   filename[fnl - sizeof(AUTH_FILE)] == '/' ) )
+	return 0;
+
+    /* Check for an auth file in the same directory.  We can't do an actual
+    ** auth password check here because CGI programs are not given the
+    ** authorization header, for security reasons.  So instead we just
+    ** prohibit access to all auth-protected files.
+    */
+    dirname = strdup( filename );
+    if ( dirname == (char*) 0 )
+	return 0;	/* out of memory */
+    cp = strrchr( dirname, '/' );
+    if ( cp == (char*) 0 )
+	(void) strcpy( dirname, "." );
+    else
+	*cp = '\0';
+    authname = malloc( strlen( dirname ) + 1 + sizeof(AUTH_FILE) );
+    if ( authname == (char*) 0 )
+	return 0;	/* out of memory */
+    (void) sprintf( authname, "%s/%s", dirname, AUTH_FILE );
+    r = stat( authname, &sb );
+    free( dirname );
+    free( authname );
+    if ( r == 0 )
+	return 0;
+#endif /* AUTH_FILE */
+
+    /* Ensure that we are not reading a CGI file. */
+    if ( cgi_pattern != (char*) 0 && match( cgi_pattern, filename ) )
+	return 0;
+
+    return 1;
+    }
+
+
 static void
 show_time( time_t t, int gmt )
     {
@@ -264,6 +333,7 @@ do_config( char* vfilename, char* filename, FILE* fp, char* directive, char* tag
     **     (formatted as 1,234,567), or abbrev for an abbreviated version
     **     displaying the number of kilobytes or megabytes the file occupies.
     */
+
     if ( strcmp( tag, "timefmt" ) == 0 )
 	{
 	(void) strncpy( timefmt, val, sizeof(timefmt) - 1 );
@@ -291,11 +361,18 @@ do_include( char* vfilename, char* filename, FILE* fp, char* directive, char* ta
     FILE* fp2;
 
     /* Inserts the text of another document into the parsed document. */
+
     if ( get_filename(
 	     vfilename, filename, directive, tag, val, filename2,
 	     sizeof(filename2) ) < 0 )
 	return;
 
+    if ( ! check_filename( filename2 ) )
+	{
+	not_permitted( directive, tag, filename2 );
+	return;
+	}
+
     fp2 = fopen( filename2, "r" );
     if ( fp2 == (FILE*) 0 )
 	{
@@ -343,6 +420,7 @@ do_echo( char* vfilename, char* filename, FILE* fp, char* directive, char* tag,
     ** printed subject to the currently configured timefmt.  The only valid
     ** tag is var, whose value is the name of the variable you wish to echo.
     */
+
     if ( strcmp( tag, "var" ) != 0 )
 	unknown_tag( filename, directive, tag );
     else
@@ -401,6 +479,7 @@ do_fsize( char* vfilename, char* filename, FILE* fp, char* directive, char* tag,
     char filename2[1000];
 
     /* Prints the size of the specified file. */
+
     if ( get_filename(
 	     vfilename, filename, directive, tag, val, filename2,
 	     sizeof(filename2) ) < 0 )
@@ -420,6 +499,7 @@ do_flastmod( char* vfilename, char* filename, FILE* fp, char* directive, char* t
     char filename2[1000];
 
     /* Prints the last modification date of the specified file. */
+
     if ( get_filename(
 	     vfilename, filename, directive, tag, val, filename2,
 	     sizeof(filename2) ) < 0 )
@@ -534,9 +614,6 @@ slurp( char* vfilename, char* filename, FILE* fp )
     int state;
     int ich;
 
-    /* First, write out a close-comment sequence. */
-    (void) fputs( "-->", stdout );
-
     /* Now slurp in the rest of the comment from the input file. */
     i = 0;
     state = ST_GROUND;
@@ -587,25 +664,25 @@ read_file( char* vfilename, char* filename, FILE* fp )
 	    {
 	    case ST_GROUND:
 	    if ( ich == '<' )
-		state = ST_LESSTHAN;
+		{ state = ST_LESSTHAN; continue; }
 	    break;
 	    case ST_LESSTHAN:
 	    if ( ich == '!' )
-		state = ST_BANG;
+		{ state = ST_BANG; continue; }
 	    else
-		state = ST_GROUND;
+		{ state = ST_GROUND; putchar( '<' ); }
 	    break;
 	    case ST_BANG:
 	    if ( ich == '-' )
-		state = ST_MINUS1;
+		{ state = ST_MINUS1; continue; }
 	    else
-		state = ST_GROUND;
+		{ state = ST_GROUND; (void) fputs ( "<!", stdout ); }
 	    break;
 	    case ST_MINUS1:
 	    if ( ich == '-' )
-		state = ST_MINUS2;
+		{ state = ST_MINUS2; continue; }
 	    else
-		state = ST_GROUND;
+		{ state = ST_GROUND; (void) fputs ( "<!-", stdout ); }
 	    break;
 	    case ST_MINUS2:
 	    if ( ich == '#' )
@@ -615,7 +692,7 @@ read_file( char* vfilename, char* filename, FILE* fp )
 		continue;
 		}
 	    else
-		state = ST_GROUND;
+		{ state = ST_GROUND; (void) fputs ( "<!--", stdout ); }
 	    break;
 	    }
 	putchar( (char) ich );
@@ -668,6 +745,12 @@ main( int argc, char** argv )
 	exit( 1 );
 	}
 
+    if ( ! check_filename( path_translated ) )
+	{
+	not_permitted( "initial", "PATH_TRANSLATED", path_translated );
+	exit( 1 );
+	}
+
     /* Open it. */
     fp = fopen( path_translated, "r" );
     if ( fp == (FILE*) 0 )
