@@ -207,24 +207,26 @@ mmc_map( char* filename, struct stat* sbP )
 
 
 void
-mmc_unmap( void* addr, struct timeval* nowP )
+mmc_unmap( void* addr, struct timeval* nowP, struct stat* sbP )
     {
     Map* m;
 
-    /* Find the Map entry for this address. */
-    for ( m = maps; m != (Map*) 0; m = m->next )
+    /* Find the Map entry for this address, either by hashing or searching. */
+    if ( sbP != (struct stat*) 0 )
+	m = find_hash( sbP->st_ino, sbP->st_dev, sbP->st_size, sbP->st_mtime );
+    else
+	for ( m = maps; m != (Map*) 0; m = m->next )
+	    if ( m->addr == addr )
+		break;
+    if ( m != (Map*) 0 )
 	{
-	if ( m->addr == addr )
-	    {
-	    --m->refcount;
-	    if ( nowP != (struct timeval*) 0 )
-		m->reftime = nowP->tv_sec;
-	    else
-		m->reftime = time( (time_t*) 0 );
-	    return;
-	    }
+	--m->refcount;
+	if ( nowP != (struct timeval*) 0 )
+	    m->reftime = nowP->tv_sec;
+	else
+	    m->reftime = time( (time_t*) 0 );
+	return;
 	}
-    /* Didn't find it.  Shrug. */
     }
 
 
