@@ -88,6 +88,8 @@ typedef struct {
 static throttletab* throttles;
 static int numthrottles, maxthrottles;
 
+#define THROTTLE_NOLIMIT 1234567890L	/* default limit (if none) */
+
 
 typedef struct {
     int conn_state;
@@ -392,6 +394,9 @@ main( int argc, char** argv )
     (void) signal( SIGHUP, handle_hup );
     (void) signal( SIGUSR1, handle_usr1 );
 
+    /* Initialize the timer package. */
+    tmr_init();
+
     /* Initialize the HTTP layer.  Got to do this before giving up root,
     ** so that we can bind to a privileged port.
     */
@@ -405,12 +410,12 @@ main( int argc, char** argv )
 
     /* Set up the occasional timer. */
     (void) tmr_create(
-	(struct timeval*) 0, occasional, (ClientData) 0,
+	(struct timeval*) 0, occasional, JunkClientData,
 	OCCASIONAL_TIME * 1000L, 1 );
 #ifdef STATS_TIME
     /* Set up the stats timer. */
     (void) tmr_create(
-	(struct timeval*) 0, show_stats, (ClientData) 0, STATS_TIME * 1000L,
+	(struct timeval*) 0, show_stats, JunkClientData, STATS_TIME * 1000L,
 	1 );
     stats_time = time( (time_t*) 0 );
     stats_connections = stats_bytes = 0L;
@@ -1141,7 +1146,7 @@ shut_down( void )
     struct timeval tv;
 
 #ifdef STATS_TIME
-    show_stats( (ClientData) 0, (struct timeval*) 0 );
+    show_stats( JunkClientData, (struct timeval*) 0 );
 #endif /* STATS_TIME */
     (void) gettimeofday( &tv, (struct timezone*) 0 );
     for ( cnum = 0; cnum < maxconnects; ++cnum )
@@ -1453,21 +1458,24 @@ handle_send( connecttab* c, struct timeval* tvP )
     if ( c->wouldblock_delay > MIN_WOULDBLOCK_DELAY )
 	c->wouldblock_delay -= MIN_WOULDBLOCK_DELAY;
 
-    /* Check if we're sending faster than the throttle. */
-    elapsed = tvP->tv_sec - c->started_at;
-    if ( elapsed == 0 || c->bytes_sent / elapsed > c->limit )
+    /* If we're throttling, check if we're sending too fast. */
+    if ( c->limit != THROTTLE_NOLIMIT )
 	{
-	c->conn_state = CNST_PAUSING;
-	fdwatch_recompute = 1;
-	/* When should we send the next c->limit bytes
-	** to get back on schedule?  If less than a second
-	** (integer math rounding), use 1/8 second.
-	*/
-	coast = ( c->bytes_sent + c->limit ) / c->limit - elapsed;
-	client_data.p = c;
-	c->wakeup_timer = tmr_create(
-	    tvP, wakeup_connection, client_data,
-	    coast ? ( coast * 1000L ) : 125L, 0 );
+	elapsed = tvP->tv_sec - c->started_at;
+	if ( elapsed == 0 || c->bytes_sent / elapsed > c->limit )
+	    {
+	    c->conn_state = CNST_PAUSING;
+	    fdwatch_recompute = 1;
+	    /* When should we send the next c->limit bytes
+	    ** to get back on schedule?  If less than a second
+	    ** (integer math rounding), use 1/8 second.
+	    */
+	    coast = ( c->bytes_sent + c->limit ) / c->limit - elapsed;
+	    client_data.p = c;
+	    c->wakeup_timer = tmr_create(
+		tvP, wakeup_connection, client_data,
+		coast ? ( coast * 1000L ) : 125L, 0 );
+	    }
 	}
     }
 
@@ -1493,7 +1501,7 @@ check_throttles( connecttab* c )
     int tnum;
 
     c->numtnums = 0;
-    c->limit = 1234567890L;
+    c->limit = THROTTLE_NOLIMIT;
     for ( tnum = 0; tnum < numthrottles && c->numtnums < MAXTHROTTLENUMS;
 	  ++tnum )
 	if ( match( throttles[tnum].pattern, c->hc->expnfilename ) )
